
/******************************************************************************
* MODULE     : associator.gen.h
* DESCRIPTION: used for translations and analysing text
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <resource.gen.h>
#include <path.gen.h>
#include <hashset.gen.h>

#module associator
#import string
#import array (string)
#import hashmap (string, int)
#import path
#import list (path)
#import hashmap (path, list<path>)
#import hashset (string)
#import resource (associator_rep, associator)

#define concept int
#define concepts list<int>
#define hashmap<string,concept> hashmap<string,int>
#define link path
#define links list<path>
#define hashmap<link,links> hashmap<path,list<path>>

/******************************************************************************
* The associator structure
******************************************************************************/

class associator_rep: public rep<associator> {
  array<string>            name;
  hashmap<string,concept>  code;
  hashmap<string,concept>  var;
  hashmap<link,links>      ass;
  links                    vals;

  string                   s;
  int                      pos;
  hashset<string>          loaded_files;

private:
  concept  new_var (string s);
  concept  new_string (string s);
  concept  get_concept (tree t);
  void     insert (link lk); 
  links    condition (tree vars, link vals, tree cond);
  links    execute_conditional (tree vars, tree conds);
  concepts execute_declarative (tree decls);

  void     skip_spaces ();
  string   read_string ();
  string   read_id ();
  bool     upcoming (string next);
  tree     parse ();

public:
  associator_rep (string s);

  void     load (string fname);
  concepts execute (string prg);
  string   meaning (concept con);
  string   translate (string s, string source_lan, string dest_lan);
};

/******************************************************************************
* Extern associators and routines for associators
******************************************************************************/

extern associator std_associator;
associator new_associator (string name);

#endmodule // associator
