
/******************************************************************************
* MODULE     : int_mod.gen.cc
* DESCRIPTION: modular integers
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Arithmetic/int_mod.gen.h>

#module code_int_mod (p)
#import int_mod (p)

/******************************************************************************
* conversion
******************************************************************************/

int_mod<p>::operator string () {
  return as_string (i);
}

int_mod<p>::operator tree () {
  return tree (as_string (i));
}

ostream&
operator << (ostream& out, int_mod<p> i) {
  return out << (i.int_mod<p>::operator string ());
}

/******************************************************************************
* basic arithmetics
******************************************************************************/

int_mod<p>
operator + (int_mod<p> i, int_mod<p> j) {
  return int_mod<p> (i.i+j.i);
}

int_mod<p>
operator - (int_mod<p> i) {
  return int_mod<p> (p-i.i);
}

int_mod<p>
operator - (int_mod<p> i, int_mod<p> j) {
  return int_mod<p> (i.i+(p-j.i));
}

int_mod<p>
operator * (int_mod<p> i, int_mod<p> j) {
  return int_mod<p> (((long long int) i.i)*j.i);
}

static void
Bezout<p> (int i, int j, int& a, int& b) {
  if (i==0) fatal_error ("division by zero", "Bezout<p>");
  if (i==1) { a=1; b=0; return; }
  Bezout<p> (j%i, i, b, a);
  a -= (j/i)*b;
}

int_mod<p>
operator / (int_mod<p> i, int_mod<p> j) {
  // cout << "\nDivide " << i << " by " << j << "\n";
  int a, b;
  Bezout<p> (j.i, p, a, b);
  if (a<0) a += p;
  return int_mod<p> (i.i*((long long int) a));
}

#endmodule // code_int_mod (p)
