
/******************************************************************************
* MODULE     : mould.gen.cc
* DESCRIPTION: abstract power mould
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Mould/mould.gen.h>
#include <hashmap.gen.cc>
#include <Mould/Mould/word.gen.cc>

#module code_mould (C, G)
#import mould (C, G)
#import code_word (G)
#import code_hashmap (word<G>, C)

/******************************************************************************
* General routines
******************************************************************************/

C
mould<C,G>::operator [] (word<G> w) {
  if (!rep->coeff->contains (w))
    rep->coeff (w)= rep->compute (w);
  return rep->coeff [w];
}

static mould<C,G>_precision= 5;

void
mould<C,G>_set_precision (int n) {
  mould<C,G>_precision= n;
}

ostream&
operator << (ostream& out, mould<C,G> M) {
  int d, i;
  for (d=0; d<mould<C,G>_precision; d++) {
    array<word<G>> a= word<G>_homogeneous (d);
    for (i=0; i<N(a); i++)
      out << a[i] << " -> " << M[a[i]] << "\n";
  }
  return out;
}

/******************************************************************************
* Constant moulds
******************************************************************************/

struct constant_mould_rep<C,G>: public mould_rep<C,G> {
  C c;
  constant_mould_rep<C,G> (C c, word<G> w) { coeff(w)= c; }
  C compute (word<G> w) { (void) w; return 0; }
};

mould<C,G>::mould<C,G> (C c, word<G> w) {
  rep= new constant_mould_rep<C,G> (c,w);
  rep->ref_count++;
}

/******************************************************************************
* Particular moulds
******************************************************************************/

static C
factorial<C> (int n) {
  if (n==0) return 1; else return n*factorial<C> (n-1);
}

struct zexp_mould_rep<C,G>: public mould_rep<C,G> {
  C c;
  zexp_mould_rep<C,G> () { word<G> empty; coeff (empty) = 0; }
  C compute (word<G> w) { return 1/factorial<C>(N(w)); }
};

mould<C,G>
mould<C,G>_zexp () {
  return new zexp_mould_rep<C,G> ();
}

struct zlog_mould_rep<C,G>: public mould_rep<C,G> {
  C c;
  zlog_mould_rep<C,G> () { word<G> empty; coeff (empty) = 0; }
  C compute (word<G> w) { return (((N(w)&1)==0)? -1/N(w): 1/N(w)); }
};

mould<C,G>
mould<C,G>_zlog () {
  return new zlog_mould_rep<C,G> ();
}

#endmodule // mould (C, G)
